<?php

namespace X_Currency\App\Controllers;

use X_Currency\App\Utils\Currency;

defined('ABSPATH') || exit;

class Geoip_Controller
{
    public function save_geo_locations($data)
    {
        try {
            $currency_id  = sanitize_key($data['currency_id']);
            if (isset($data['geo_countries_type']) && isset($data['disable_countries'])) {
                $country_list = serialize($data['disable_countries']);
                update_post_meta($currency_id, 'geo_countries_type', sanitize_text_field($data['geo_countries_type']));
                update_post_meta($currency_id, 'disable_countries', $country_list);
            } else {
                update_post_meta($currency_id, 'geo_countries_type', "enable");
                update_post_meta($currency_id, 'disable_countries', serialize([]));
            }
            update_post_meta($currency_id, 'welcome_country', $data['welcome_country']);

            return [
                'message' => esc_html__('Currency geo location updated successfully!', 'x-currency'),
                'status'  => 'success'
            ];
        } catch (\Exception $e) {
            return Currency::failed_response();
        }
    }

    public function geo_input_fields()
    {
        $countries    = new \WC_Countries;
        $country_list = [];

        foreach ($countries->get_countries() as $key => $country) {
            $country_list[] = ['value' => $key, 'label' => $country];
        }

        return [
            'status' => 'success',
            'data'   => [
                'geo_countries_type' => [
                    'label'   => esc_html__('Country List Type', 'x-currency'),
                    'type'    => 'XCurrencySelect',
                    'options' => [
                        ['value' => 'enable', 'label' => esc_html__('Enable', 'x-currency')],
                        ['value' => 'disable', 'label' => esc_html__('Disable', 'x-currency')]
                    ],
                    'default' => 'disable',
                    'help'    => esc_html__("This feature will work if geo location is enable", 'x-currency')
                ],

                'disable_countries'  => [
                    'type'    => 'XCurrencyMultiSelect',
                    'label'   => esc_html__('Country List', 'x-currency'),
                    'options' => $country_list,
                    'help'    => esc_html__("The countries you select here, will be disabled/enabled based on the 'Country list type' field", 'x-currency')
                ],

                'welcome_country'    => [
                    'label'       => esc_html__('Welcome Country', 'x-currency'),
                    'type'        => 'XCurrencySelect',
                    'isClearable' => true,
                    'options'     => $country_list,
                    'help'        => esc_html__("This feature will work if your user's welcome currency is auto", 'x-currency')
                ]
            ]
        ];
    }

}
