<?php

namespace X_Currency\App\Controllers;

use X_Currency;
use X_Currency\App\Controllers\Currency_Controller;
use X_Currency\App\Utils\Currency;
use X_Currency\App\Utils\Switcher;
use X_Currency\App\Core\Singleton;

defined( 'ABSPATH' ) || exit;

class Switcher_Controller
{
    use Singleton;

    public function switcher_list()
    {
        return [
            'message' => esc_html__( 'Switcher List Retrieved Successfully!', 'x-currency' ),
            'data'    => Switcher::switcher_list_data( x_currency_switcher_list() ),
            'status'  => 'success'
        ];
    }

    public function get_pages()
    {
        $pages = [
            ['value' => 'all', 'label' => 'All Page'],
            ['value' => 'home', 'label' => 'Home'],
            ['value' => 'wc_archive', 'label' => 'WooCommerce Archive'],
            ['value' => 'wc_single', 'label' => 'WooCommerce Single'],
            ['value' => 'post_single', 'label' => 'Post Single'],
        ];
        foreach ( get_pages() as $page ) {
            array_push( $pages, ['value' => $page->ID, 'label' => $page->post_title] );
        }
        return ['data' => $pages];
    }

    /**
     * @return mixed
     */
    public function side_stick_switcher_list()
    {
        $posts = get_posts( [
            'post_type'   => X_Currency::POST_TYPE_SWITCHER,
            'post_status' => 'publish',
            'numberposts' => -1,
            'meta_query'  => [
                [
                    'key'     => 'type',
                    'value'   => 'sticky',
                    'compare' => '='
                ]
            ]
        ] );
        return Switcher::switcher_list_data( $posts );
    }

    /**
     * @param $data
     */
    public function create_switcher( $data )
    {
        try {
            unset( $data['action'] );
            $args = [
                'post_title'  => sanitize_text_field( $data['name'] ),
                'post_type'   => X_Currency::POST_TYPE_SWITCHER,
                'post_status' => $data['active'] == true ? 'publish' : 'draft'
            ];

            $post_id = wp_insert_post( $args );
            unset( $data['name'] );
            unset( $data['active'] );

            // save custom css
            if ( isset( $data['custom_css'] ) ) {
                add_post_meta( $post_id, 'custom_css', $data['custom_css'] );
                unset( $data['custom_css'] );
            }

            foreach ( $data as $key => $value ) {
                add_post_meta( $post_id, sanitize_key( $key ), sanitize_text_field( $value ) );
            }

            return [
                'message' => esc_html__( 'Switcher Created Successfully!', 'x-currency' ),
                'data'    => [
                    'id'         => $post_id,
                    'short_code' => "[" . X_Currency::POST_TYPE_SWITCHER . " id=" . $post_id . "]"
                ],
                'status'  => 'success'
            ];

        } catch ( \Exception $e ) {
            return Currency::failed_response();
        }
    }

    /**
     * @param $data
     */
    public function update_switcher( $data )
    {
        try {
            $post_id = $data['id'];

            $args = [
                'ID'          => sanitize_key( $data['id'] ),
                'post_title'  => sanitize_text_field( $data['name'] ),
                'post_type'   => X_Currency::POST_TYPE_SWITCHER,
                'post_status' => $data['active'] == true ? 'publish' : 'draft'
            ];

            wp_update_post( $args );

            unset( $data['id'] );
            unset( $data['name'] );
            unset( $data['active'] );

            if ( isset( $data['custom_css'] ) ) {
                update_post_meta( $post_id, 'custom_css', $data['custom_css'] );
                unset( $data['custom_css'] );
            }

            foreach ( $data as $key => $value ) {
                update_post_meta( $post_id, sanitize_key( $key ), sanitize_text_field( $value ) );
            }

            return [
                'message' => esc_html__( 'Switcher Updated Successfully!', 'x-currency' ),
                'data'    => [
                    'id'         => $post_id,
                    'short_code' => "[" . X_Currency::POST_TYPE_SWITCHER . " id=" . $post_id . "]"
                ],
                'status'  => 'success'
            ];

        } catch ( \Exception $e ) {
            return Currency::failed_response();
        }
    }

    /**
     * @param $data
     */
    public function switcher_organizer( $data )
    {
        try {
            $ids = $data['keys'];
            ( new Currency_Controller )->query( $ids, $data['type'], false );
            return [
                'message' => esc_html__( 'Switcher organized successfully!', 'x-currency' ),
                'status'  => 'success'
            ];
        } catch ( \Exception $e ) {
            return Currency::failed_response();
        }
    }
}
