<?php

use X_Currency\App\Controllers\Settings_Controller;
use X_Currency\App\Utils\Currency;
use X_Currency\App\Utils\Settings;

function x_currency_global_settings()
{
    $settingInputs = Settings_Controller::inputs();

    $dbSettings = get_option( Settings::SETTINGS_OPTION_KEY );

    if ( false !== $dbSettings ) {
        $dbSettings = unserialize( $dbSettings );
    } else {
        $dbSettings = [];
    }

    $globalSettings = [];

    foreach ( $settingInputs as $key => $input ) {
        if ( isset( $dbSettings[$key] ) ) {
            $globalSettings[$key] = $dbSettings[$key];
        } elseif ( isset( $input['default'] ) ) {
            $globalSettings[$key] = $input['default'];
        } else {
            $globalSettings[$key] = '';
        }
    }

    return $globalSettings;
}

if ( !function_exists( 'x_currency_get_json_file_content' ) ) {

    /**
     * @param $file_url
     */
    function x_currency_get_json_file_content( $file_url )
    {
        $fonts_string = file_get_contents( $file_url );
        return json_decode( $fonts_string, true );
    }
}

if ( !function_exists( 'x_currency_get_currency_id_by_meta' ) ) {

    /**
     * @param $key
     * @param $value
     * @return mixed
     */
    function x_currency_get_currency_id_by_meta( $key, $value )
    {
        $currencies = x_currency_get_currencies_by_meta( $key, $value );

        if ( isset( $currencies[0] ) ) {
            return $currencies[0]->ID;
        }
        return false;
    }
}

if ( !function_exists( 'x_currency_get_currencies_by_meta' ) ) {

    /**
     * @param $key
     * @param $value
     * @param $post_status
     */
    function x_currency_get_currencies_by_meta( $key, $value, $post_status = 'publish' )
    {
        return get_posts( [
            'meta_key'    => $key,
            'meta_value'  => $value,
            'post_type'   => X_Currency::POST_TYPE,
            'post_status' => $post_status
        ] );
    }
}

if ( !function_exists( 'x_currency_get_currency_by_id' ) ) {

    /**
     * @param $currency_id
     */
    function x_currency_get_currency_by_id( $currency_id )
    {
        $flag = (int) get_post_meta( $currency_id, 'flag', true );

        $country_list = get_post_meta( $currency_id, 'disable_countries', true );

        if ( false != $country_list ) {
            $country_list = unserialize( $country_list );
        } else {
            $country_list = [];
        }

        $geo_countries_type = get_post_meta( $currency_id, 'geo_countries_type', true );

        return [
            "code"               => get_post_meta( $currency_id, 'code', true ),
            "symbol"             => get_post_meta( $currency_id, 'symbol', true ),
            "rate"               => get_post_meta( $currency_id, 'rate', true ),
            "rate_type"          => get_post_meta( $currency_id, 'rate_type', true ), // fixed || auto
            "extra_fee"          => get_post_meta( $currency_id, 'extra_fee', true ),
            "extra_fee_type"     => get_post_meta( $currency_id, 'extra_fee_type', true ), // fixed || percent
            "payment_gateways"   => get_post_meta( $currency_id, 'payment_gateways', true ),
            "thousand_separator" => get_post_meta( $currency_id, 'thousand_separator', true ),
            'decimal'            => (int) get_post_meta( $currency_id, 'decimal', true ),
            'decimal_separator'  => get_post_meta( $currency_id, 'decimal_separator', true ),
            'position'           => get_post_meta( $currency_id, 'position', true ),
            'geo_countries_type' => empty( $geo_countries_type ) ? 'disable' : $geo_countries_type,
            'disable_countries'  => $country_list,
            'welcome_country'    => get_post_meta( $currency_id, 'welcome_country', true ),
            'flag'               => $flag
        ];
    }
}

if ( !function_exists( 'x_currency_exchange' ) ) {

    /**
     * @param $price
     * @return mixed
     */
    function x_currency_exchange( $price )
    {
        if ( !empty( $price ) ) {
            global $x_currency;
            $price = $price * $x_currency['selected_currency']['rate'];
        }
        return $price;
    }
}

if ( !function_exists( 'x_currency_product_extra_price' ) ) {

    /**
     * @param $price
     * @return mixed
     */
    function x_currency_product_extra_price( $price )
    {
        global $x_currency;
        if ( !empty( $x_currency['selected_currency']['extra_fee'] ) ) {
            if ( $x_currency['selected_currency']['extra_fee_type'] === 'percent' ) {
                $extra_fee = ( $x_currency['selected_currency']['extra_fee'] / 100 ) * $price;
            } else {
                $extra_fee = $x_currency['selected_currency']['extra_fee'];
            }

            return $price + $extra_fee;
        }

        return $price;
    }
}

if ( !function_exists( 'x_currency_price_format' ) ) {

    /**
     * @param $settings
     * @return mixed
     */
    function x_currency_price_format( $settings )
    {
        switch ( $settings['position'] ) {
            case 'right':
                $format = '%2$s%1$s';
                break;
            case 'left_space':
                $format = '%1$s&nbsp;%2$s';
                break;
            case 'right_space':
                $format = '%2$s&nbsp;%1$s';
                break;
            default:
                $format = '%1$s%2$s';
        }
        return $format;
    }
}

if ( !function_exists( 'x_currency_switcher_list' ) ) {
    function x_currency_switcher_list( string $type = 'all', string $post_status = 'publish, draft' )
    {
        $args = [
            'post_type'   => X_Currency::POST_TYPE_SWITCHER,
            'post_status' => $post_status,
            'numberposts' => -1,
            'order'       => 'ASC'
        ];

        if ( $type !== 'all' ) {
            $args['meta_query'] = [
                [
                    'key'     => 'type',
                    'value'   => $type,
                    'compare' => '='
                ]
            ];
        }

        return get_posts( $args );
    }
}

if ( !function_exists( 'x_currency_price_decimal' ) ) {

    /**
     * @return mixed
     */
    function x_currency_price_decimal()
    {
        global $x_currency;
        if ( isset( $x_currency['global_settings']['prices_without_cents'] ) && $x_currency['global_settings']['prices_without_cents'] == true ) {
            return 0;
        }
        return $x_currency['selected_currency']['decimal'];
    }
}

if ( !function_exists( 'x_currency_get_flags_src' ) ) {

    /**
     * @return mixed
     */
    function x_currency_get_flags_src()
    {
        $sort_ids   = Currency::sort_ids();
        $attachment = [];

        foreach ( $sort_ids as $id ) {
            $flag                 = (int) get_post_meta( $id, 'flag', true );
            $attachment_image_src = wp_get_attachment_image_src( $flag );

            if ( empty( $attachment_image_src ) ) {
                $attachment[] = [X_CURRENCY_URL . 'media/common/dummy-flag.jpg'];
            } else {
                $attachment[] = $attachment_image_src;
            }
        }
        return $attachment;
    }
}

if ( !function_exists( 'x_currency_get_currency_rate_by_code' ) ) {
    /**
     * @param $currency_code
     * @return mixed
     */
    function x_currency_get_currency_rate_by_code( $currency_code )
    {
        $currencies = x_currency_get_currencies_by_meta( 'code', $currency_code );
        if ( empty( $currencies[0] ) ) {
            return false;
        }

        global $x_currency;

        $key = array_search($currencies[0]->ID, array_column($x_currency['currencies'], 'id'));

        if(!is_int($key)) {
            return false;
        }

        return $x_currency['currencies'][$key]['rate'];
    }
}

if ( !function_exists( 'x_currency_get_base_currency_code' ) ) {
    /**
     * @return mixed
     */
    function x_currency_get_base_currency_code()
    {
        global $x_currency;

        $key = array_search($x_currency['global_settings']['base_currency'], array_column($x_currency['currencies'], 'id'));

        if(!is_int($key)) {
            return false;
        }
        return $x_currency['currencies'][$key]['code'];
    }
}

function x_currency_symbols()
{
    return [
        'AED' => '&#x62f;.&#x625;',
        'AFN' => '&#x60b;',
        'ALL' => 'L',
        'AMD' => 'AMD',
        'ANG' => '&fnof;',
        'AOA' => 'Kz',
        'ARS' => '&#36;',
        'AUD' => '&#36;',
        'AWG' => 'Afl.',
        'AZN' => 'AZN',
        'BAM' => 'KM',
        'BBD' => '&#36;',
        'BDT' => '&#2547;&nbsp;',
        'BGN' => '&#1083;&#1074;.',
        'BHD' => '.&#x62f;.&#x628;',
        'BIF' => 'Fr',
        'BMD' => '&#36;',
        'BND' => '&#36;',
        'BOB' => 'Bs.',
        'BRL' => '&#82;&#36;',
        'BSD' => '&#36;',
        'BTC' => '&#3647;',
        'BTN' => 'Nu.',
        'BWP' => 'P',
        'BYR' => 'Br',
        'BYN' => 'Br',
        'BZD' => '&#36;',
        'CAD' => '&#36;',
        'CDF' => 'Fr',
        'CHF' => '&#67;&#72;&#70;',
        'CLP' => '&#36;',
        'CNY' => '&yen;',
        'COP' => '&#36;',
        'CRC' => '&#x20a1;',
        'CUC' => '&#36;',
        'CUP' => '&#36;',
        'CVE' => '&#36;',
        'CZK' => '&#75;&#269;',
        'DJF' => 'Fr',
        'DKK' => 'DKK',
        'DOP' => 'RD&#36;',
        'DZD' => '&#x62f;.&#x62c;',
        'EGP' => 'EGP',
        'ERN' => 'Nfk',
        'ETB' => 'Br',
        'EUR' => '&euro;',
        'FJD' => '&#36;',
        'FKP' => '&pound;',
        'GBP' => '&pound;',
        'GEL' => '&#x20be;',
        'GGP' => '&pound;',
        'GHS' => '&#x20b5;',
        'GIP' => '&pound;',
        'GMD' => 'D',
        'GNF' => 'Fr',
        'GTQ' => 'Q',
        'GYD' => '&#36;',
        'HKD' => '&#36;',
        'HNL' => 'L',
        'HRK' => 'kn',
        'HTG' => 'G',
        'HUF' => '&#70;&#116;',
        'IDR' => 'Rp',
        'ILS' => '&#8362;',
        'IMP' => '&pound;',
        'INR' => '&#8377;',
        'IQD' => '&#x62f;.&#x639;',
        'IRR' => '&#xfdfc;',
        'IRT' => '&#x062A;&#x0648;&#x0645;&#x0627;&#x0646;',
        'ISK' => 'kr.',
        'JEP' => '&pound;',
        'JMD' => '&#36;',
        'JOD' => '&#x62f;.&#x627;',
        'JPY' => '&yen;',
        'KES' => 'KSh',
        'KGS' => '&#x441;&#x43e;&#x43c;',
        'KHR' => '&#x17db;',
        'KMF' => 'Fr',
        'KPW' => '&#x20a9;',
        'KRW' => '&#8361;',
        'KWD' => '&#x62f;.&#x643;',
        'KYD' => '&#36;',
        'KZT' => '&#8376;',
        'LAK' => '&#8365;',
        'LBP' => '&#x644;.&#x644;',
        'LKR' => '&#xdbb;&#xdd4;',
        'LRD' => '&#36;',
        'LSL' => 'L',
        'LYD' => '&#x644;.&#x62f;',
        'MAD' => '&#x62f;.&#x645;.',
        'MDL' => 'MDL',
        'MGA' => 'Ar',
        'MKD' => '&#x434;&#x435;&#x43d;',
        'MMK' => 'Ks',
        'MNT' => '&#x20ae;',
        'MOP' => 'P',
        'MRU' => 'UM',
        'MUR' => '&#x20a8;',
        'MVR' => '.&#x783;',
        'MWK' => 'MK',
        'MXN' => '&#36;',
        'MYR' => '&#82;&#77;',
        'MZN' => 'MT',
        'NAD' => 'N&#36;',
        'NGN' => '&#8358;',
        'NIO' => 'C&#36;',
        'NOK' => '&#107;&#114;',
        'NPR' => '&#8360;',
        'NZD' => '&#36;',
        'OMR' => '&#x631;.&#x639;.',
        'PAB' => 'B/.',
        'PEN' => 'S/',
        'PGK' => 'K',
        'PHP' => '&#8369;',
        'PKR' => '&#8360;',
        'PLN' => '&#122;&#322;',
        'PRB' => '&#x440;.',
        'PYG' => '&#8370;',
        'QAR' => '&#x631;.&#x642;',
        'RMB' => '&yen;',
        'RON' => 'lei',
        'RSD' => '&#1088;&#1089;&#1076;',
        'RUB' => '&#8381;',
        'RWF' => 'Fr',
        'SAR' => '&#x631;.&#x633;',
        'SBD' => '&#36;',
        'SCR' => '&#x20a8;',
        'SDG' => '&#x62c;.&#x633;.',
        'SEK' => '&#107;&#114;',
        'SGD' => '&#36;',
        'SHP' => '&pound;',
        'SLL' => 'Le',
        'SOS' => 'Sh',
        'SRD' => '&#36;',
        'SSP' => '&pound;',
        'STN' => 'Db',
        'SYP' => '&#x644;.&#x633;',
        'SZL' => 'E',
        'THB' => '&#3647;',
        'TJS' => '&#x405;&#x41c;',
        'TMT' => 'm',
        'TND' => '&#x62f;.&#x62a;',
        'TOP' => 'T&#36;',
        'TRY' => '&#8378;',
        'TTD' => '&#36;',
        'TWD' => '&#78;&#84;&#36;',
        'TZS' => 'Sh',
        'UAH' => '&#8372;',
        'UGX' => 'UGX',
        'USD' => '&#36;',
        'UYU' => '&#36;',
        'UZS' => 'UZS',
        'VEF' => 'Bs F',
        'VES' => 'Bs.S',
        'VND' => '&#8363;',
        'VUV' => 'Vt',
        'WST' => 'T',
        'XAF' => 'CFA',
        'XCD' => '&#36;',
        'XOF' => 'CFA',
        'XPF' => 'Fr',
        'YER' => '&#xfdfc;',
        'ZAR' => '&#82;',
        'ZMW' => 'ZK'
    ];
}
