<?php

namespace X_Currency\App\Service_Providers;

use WC_Coupon;
use X_Currency\App\Compatibility\Compatibility;
use X_Currency\App\Supports\Woocommerce\Approximate_Product_Price;
use X_Currency\App\Supports\Woocommerce\Coupon;
use X_Currency\App\Supports\Woocommerce\Order;
use X_Currency\App\Supports\Woocommerce\Product;
use X_Currency\App\Supports\Woocommerce\Report;
use X_Currency\App\Supports\Woocommerce\Shipping;
use X_Currency\Bootstrap\Service_Provider;

defined( 'ABSPATH' ) || exit;

class Woocommerce_Service_Provider implements Service_Provider
{
    /**
     * @var mixed
     */
    private $settings;
    /**
     * @var mixed
     */
    private $global_settings;
    /**
     * @var mixed
     */
    private $fixed_prices;
    /**
     * @var mixed
     */
    public $variation_price;
    /**
     * @var mixed
     */
    public $simple_product_price = false;
    /**
     * @var mixed
     */
    private $updated_prices;

    private $simple_product_specific_price = [];

    private $cart_coupon_free_shipping_check = false;

    private $is_cart_coupon_allow_free_shipping = false;

    public function boot()
    {
        add_action( 'init', [$this, 'init'] );
    }

    public function init()
    {
        global $x_currency;
        $this->settings        = $x_currency['selected_currency'];
        $this->global_settings = $x_currency['global_settings'];

        new Order();
        if ( is_admin() ) {
            new Coupon();
            new Shipping();
            new Product();
            if ( isset( $_GET['page'] ) && $_GET['page'] == 'wc-reports' ) { //phpcs:ignore WordPress.Security.NonceVerification.Recommended
                new Report();
            }
        } elseif ( is_array( $this->settings ) ) {
            add_filter( 'woocommerce_product_get_regular_price', [$this, 'product_regular_price'], 10, 2 );
            add_filter( 'woocommerce_product_get_price', [$this, 'simple_product_price'], 10, 2 );
            add_filter( 'woocommerce_product_get_sale_price', [$this, 'product_sale_price'], 10, 2 );
            add_filter( 'woocommerce_price_format', [$this, 'price_format'], 99999999 );
            add_action( 'woocommerce_coupon_loaded', [$this, 'coupon_amount'], 9999 );
            add_filter( 'woocommerce_available_payment_gateways', [$this, 'payment_gateways'] );
            add_filter( 'woocommerce_currency', [$this, 'currency_code'] );
            add_filter( 'wc_get_price_decimals', 'x_currency_price_decimal' );
            add_filter( 'woocommerce_shipping_packages', [$this, 'shippings_cost'] );
            add_filter( 'woocommerce_variation_prices', [$this, 'variation_prices'], 0, 2 );
            add_filter( 'woocommerce_product_variation_get_price', [$this, 'variation_get_price'], 9999, 2 );
            add_filter( 'woocommerce_product_variation_get_regular_price', [$this, 'variation_get_regular_price'], 9999, 2 );
            add_filter( 'wc_get_price_thousand_separator', [$this, 'price_thousand_separator'] );
            add_filter( 'wc_get_price_decimal_separator', [$this, 'price_decimal_separator'] );

            Compatibility::run();
            // cart total extra fee
            if ( isset( $this->global_settings['cart_total_extra_fee'] ) && $this->global_settings['cart_total_extra_fee'] == true ) {
                add_filter( 'woocommerce_cart_get_total', 'x_currency_product_extra_price' );
                if ( isset( $this->global_settings['cart_total_extra_fee_message'] ) && $this->global_settings['cart_total_extra_fee_message'] == true ) {
                    add_filter( 'woocommerce_cart_total', [$this, 'cart_total_extra_fee'] );
                }
            }

            if ( !empty( $this->global_settings['approximate_product_price'] ) || !empty( $this->global_settings['approximate_cart_price'] ) ) {
                new Approximate_Product_Price();
            }
        }
        add_filter( 'woocommerce_currency_symbols', [$this, 'currency_symbol'] );
    }

    /**
     * @param $price
     * @return mixed
     */
    public function product_regular_price( $price, $product )
    {
        return $this->get_product_regular_and_sale_price( $price, 'regular', $product );
    }

    /**
     * @param $price
     * @return mixed
     */
    public function product_sale_price( $price, $product )
    {
        if ( !empty( $price ) && !empty( $this->global_settings['specific_product_price'] ) ) {
            if ( $product->is_type( 'simple' ) || $product->is_type( 'external' ) ) {
                return $this->get_product_regular_and_sale_price( $price, 'sale', $product );
            }
        }
        return x_currency_exchange( $price );
    }

    public function get_product_regular_and_sale_price( $price, $price_type, $product )
    {
        $this->get_simple_product_specific_price( $product );
        $lower_currency_code = strtolower( $this->settings['code'] );
        if ( !empty( $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code][$price_type] ) ) {
            return $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code][$price_type];
        }

        return x_currency_exchange( $price );
    }

    /**
     * @param $price
     * @return mixed
     */
    public function simple_product_price( $price, $product )
    {
        $this->get_simple_product_specific_price( $product );
        $lower_currency_code = strtolower( $this->settings['code'] );
        if ( !empty( $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code]['sale'] ) ) {
            return $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code]['sale'];
        } elseif ( !empty( $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code]['regular'] ) ) {
            return $this->simple_product_specific_price[$product->get_ID()][$lower_currency_code]['regular'];
        }

        return x_currency_exchange( $price );
    }

    /**
     * @return mixed
     */
    public function price_format()
    {
        return x_currency_price_format( $this->settings );
    }

    /**
     * @return mixed
     */
    public function currency_symbol( $symbols )
    {
        global $x_currency;
        foreach ( $x_currency['currencies'] as $id => $currency ) {
            $symbols[$currency['code']] = $currency['symbol'];
        }
        return $symbols;
    }

    /**
     * @param $coupon
     * @return mixed
     */
    public function coupon_amount( $coupon )
    {
        if ( isset( $this->global_settings['specific_coupon_amount'] ) && $this->global_settings['specific_coupon_amount'] == 'true' ) {
            $coupon_for_each_currency = $coupon->get_meta( 'x_currency_coupon_amounts' );
            if ( $coupon_for_each_currency ) {
                $coupon_for_each_currency = unserialize( $coupon_for_each_currency );
                if ( !empty( $coupon_for_each_currency[$this->settings['code']] ) ) {
                    $coupon->set_amount( $coupon_for_each_currency[$this->settings['code']] );
                    return $coupon;
                }
            }
        }

        $type = $coupon->get_discount_type();
        if ( $type === 'fixed_product' || $type === 'fixed_cart' ) {
            $coupon->set_amount( x_currency_exchange( $coupon->get_amount() ) );
        }
        return $coupon;
    }

    public function get_simple_product_specific_price( $product )
    {
        if ( empty( $this->simple_product_specific_price[$product->get_ID()] ) ) {
            $this->simple_product_specific_price[$product->get_ID()] = json_decode( get_post_meta( $product->get_ID(), 'x_currency_simple', true ), true );
        }
    }
    /**
     * @param $gateways
     * @return mixed
     */
    public function payment_gateways( $gateways )
    {
        if ( is_wc_endpoint_url( 'order-pay' ) ) {
            $order                              = wc_get_order( get_query_var( 'order-pay' ) );
            $currency                           = x_currency_get_currency_id_by_meta( 'code', $order->get_currency() );
            $this->settings['payment_gateways'] = get_post_meta( $currency, 'payment_gateways', true );
        }

        if ( empty( $_SERVER['HTTP_REFERER'] ) || isset( $_SERVER['HTTP_REFERER'] ) && strpos( sanitize_text_field( wp_unslash( $_SERVER['HTTP_REFERER'] ) ), 'wp-admin' ) === false ) {
            if ( !empty( $this->settings['payment_gateways'] ) && is_array( $this->settings['payment_gateways'] ) ) {
                foreach ( $this->settings['payment_gateways'] as $gateway ) {
                    if ( isset( $gateways[$gateway] ) ) {
                        unset( $gateways[$gateway] );
                    }
                }
            }
        }
        return $gateways;
    }

    /**
     * @param $prices
     * @return mixed
     */
    public function variation_prices( $prices, $product )
    {
        $updated_prices = [];
        $currency_code  = strtolower( $this->settings['code'] );

        if ( empty( $this->global_settings['specific_product_price'] ) ) {
            foreach ( $prices as $price_type => $values ) {
                foreach ( $values as $variation_id => $price ) {
                    $updated_prices[$price_type][$variation_id] = x_currency_exchange( $price );
                }
            }
        } else {
            $variation_fixed_prices = json_decode( get_post_meta( $product->get_ID(), 'x_currency_variation', true ), true );
            foreach ( $prices as $price_type => $values ) {
                foreach ( $values as $variation_id => $price ) {
                    if ( $price_type === 'price' ) {
                        if ( !empty( $variation_fixed_prices[$variation_id][$currency_code]['sale'] ) ) {
                            $updated_price = $variation_fixed_prices[$variation_id][$currency_code]['sale'];
                        } elseif ( !empty( $variation_fixed_prices[$variation_id][$currency_code]['regular'] ) ) {
                            $updated_price = $variation_fixed_prices[$variation_id][$currency_code]['regular'];
                        } else {
                            $updated_price = x_currency_exchange( $price );
                        }
                    } else {
                        $price_type0 = str_replace( '_price', '', $price_type );
                        if ( 'sale_price' === $price_type ) {
                            if ( !empty( $variation_fixed_prices[$variation_id][$currency_code][$price_type0] ) ) {
                                $updated_price = $variation_fixed_prices[$variation_id][$currency_code][$price_type0];
                            } elseif ( !empty( $variation_fixed_prices[$variation_id][$currency_code]['regular'] ) ) {
                                $updated_price = $variation_fixed_prices[$variation_id][$currency_code]['regular'];
                            } else {
                                $updated_price = x_currency_exchange( $price );
                            }

                            if ( $updated_prices['regular_price'][$variation_id] == $updated_price ) {
                                $updated_prices['regular_price'][$variation_id] = '';
                            }
                        } else {
                            if ( !empty( $variation_fixed_prices[$variation_id][$currency_code][$price_type0] ) ) {
                                $updated_price = $variation_fixed_prices[$variation_id][$currency_code][$price_type0];
                            } else {
                                $updated_price = x_currency_exchange( $price );
                            }
                        }
                    }

                    $updated_prices[$price_type][$variation_id] = $updated_price;
                }
            }
        }

        $this->updated_prices = $updated_prices;

        return $updated_prices;
    }

    /**
     * @param $price
     * @param $product
     * @return mixed
     */
    public function variation_get_regular_price( $price, $product )
    {
        if ( empty( $this->updated_prices ) && !empty( $this->global_settings['specific_product_price'] ) ) {
            $currency_code          = strtolower( $this->settings['code'] );
            $variation_fixed_prices = json_decode( get_post_meta( $product->get_parent_id(), 'x_currency_variation', true ), true );
            if ( !empty( $variation_fixed_prices[$product->get_ID()][$currency_code]['regular'] ) ) {
                return $variation_fixed_prices[$product->get_ID()][$currency_code]['regular'];
            }
        }
        return isset( $this->updated_prices['regular_price'][$product->get_ID()] ) ? $this->updated_prices['regular_price'][$product->get_ID()] : x_currency_exchange( $price );
    }

    /**
     * @param $price
     * @param $product
     * @return mixed
     */
    public function variation_get_price( $price, $product )
    {
        if ( empty( $this->updated_prices ) && !empty( $this->global_settings['specific_product_price'] ) ) {
            $currency_code          = strtolower( $this->settings['code'] );
            $variation_fixed_prices = json_decode( get_post_meta( $product->get_parent_id(), 'x_currency_variation', true ), true );
            if ( !empty( $variation_fixed_prices[$product->get_ID()][$currency_code]['sale'] ) ) {
                return $variation_fixed_prices[$product->get_ID()][$currency_code]['sale'];
            } elseif ( !empty( $variation_fixed_prices[$product->get_ID()][$currency_code]['regular'] ) ) {
                return $variation_fixed_prices[$product->get_ID()][$currency_code]['regular'];
            }
        }
        return isset( $this->updated_prices['price'][$product->get_ID()] ) ? $this->updated_prices['price'][$product->get_ID()] : x_currency_exchange( $price );
    }

    /**
     * @return mixed
     */
    public function currency_code()
    {
        return $this->settings['code'];
    }

    /**
     * @param $packages
     * @return mixed
     */
    public function shippings_cost( $packages )
    {
        global $x_currency;

        if ( $x_currency['selected_currency']['code'] != x_currency_get_base_currency_code() ) {
            foreach ( $packages as $key => $package ) {
                foreach ( $package['rates'] as $rate ) {
                    /**
                     * if specific amount shipping enable
                     */
                    if ( isset( $this->global_settings['specific_shipping_amount'] ) && $this->global_settings['specific_shipping_amount'] == 'true' ) {
                        $settings_key             = 'woocommerce_' . $rate->get_method_id() . '_' . $rate->get_instance_id() . '_settings';
                        $shipping_method_settings = get_option( $settings_key );

                        if ( is_array( $shipping_method_settings ) ) {
                            /**
                             * if shipping method has specific amount (selected currency)
                             */
                            if ( !empty( $shipping_method_settings['x_currency_' . strtolower( $this->settings['code'] )] ) ) {
                                $specific_cost = $shipping_method_settings['x_currency_' . strtolower( $this->settings['code'] )];
                                /**
                                 * if shipping method is free shipping
                                 */
                                if ( 'free_shipping' === $rate->get_method_id() ) {
                                    if ( in_array( $shipping_method_settings['requires'], ['both', 'min_amount'] ) || ( $shipping_method_settings['requires'] === 'either' && !$this->cart_coupon_allow_free_shipping() ) ) {
                                        if ( $package['contents_cost'] < $specific_cost ) {
                                            unset( $packages[$key]['rates'][$rate->get_id()] );
                                        }
                                    }
                                } else {
                                    $rate->set_cost( $specific_cost );
                                }
                                continue;
                            } else {
                                if ( 'free_shipping' === $rate->get_method_id() ) {
                                    if ( in_array( $shipping_method_settings['requires'], ['both', 'min_amount'] ) || ( $shipping_method_settings['requires'] === 'either' && !$this->cart_coupon_allow_free_shipping() ) ) {
                                        $minimum_cart_requires = x_currency_exchange( $shipping_method_settings['min_amount'] );
                                        if ( $package['contents_cost'] < $minimum_cart_requires ) {
                                            unset( $packages[$key]['rates'][$rate->get_id()] );
                                        }
                                    }
                                    continue;
                                }
                            }
                        }
                    }
                    $rate->set_cost( x_currency_exchange( $rate->get_cost() ) );
                }
            }
        }
        return $packages;
    }

    private function cart_coupon_allow_free_shipping(): bool
    {
        if ( !$this->cart_coupon_free_shipping_check ) {
            $applied_coupons = WC()->cart->get_applied_coupons();
            foreach ( $applied_coupons as $coupon_code ) {
                $coupon = new WC_Coupon( $coupon_code );
                if ( $coupon->get_free_shipping() ) {
                    $this->is_cart_coupon_allow_free_shipping = true;
                }
            }
        }
        return $this->is_cart_coupon_allow_free_shipping;
    }

    /**
     * @return mixed
     */
    public function price_thousand_separator()
    {
        return $this->settings['thousand_separator'];
    }

    /**
     * @return mixed
     */
    public function price_decimal_separator()
    {
        return $this->settings['decimal_separator'];
    }

    /**
     * @param $view
     * @return mixed
     */
    public function cart_total_extra_fee( $view )
    {
        if ( !empty( $this->settings['extra_fee'] ) && $this->settings['extra_fee'] != 0 ) {
            if ( $this->settings['extra_fee_type'] === 'percent' ) {
                $extra_fee_text = $this->settings['extra_fee'] . ' %';
            } else {
                $extra_fee_text = sprintf( get_woocommerce_price_format(), $this->settings['symbol'], $this->settings['extra_fee'] );
            }
            $view .= '<span class="x-currency-extra-fee">' . sprintf( esc_html__( 'Extra fee %s for %s currency', 'x-currency' ), $extra_fee_text, get_woocommerce_currency() ) . '</span>';
        }
        return $view;
    }
}
