<?php

namespace X_Currency\App\Supports\Rate_Provider;

use X_Currency\App\Core\Singleton;
use X_Currency\App\Utils\Currency;
use X_Currency\App\Utils\Settings;

class Rate_Exchange
{
    use Singleton;

    /**
     * @param $data
     * @return mixed
     */
    public function exchange_single( $data )
    {
        $currency_id   = $data['currency_id'];
        $currency_code = $data['currency_code'];
        return $this->exchange_all( $currency_id, $currency_code );
    }

    /**
     * @param $currency_id
     * @param $currency_code
     * @return mixed
     */
    public function exchange_all( $currency_id = 'all', $currency_code = null )
    {
        $settings = Settings::settings_data();

        if ( !empty( $settings['default_rate_provider'] ) ) {

            $rate_provider = $settings['default_rate_provider'];
            $provider      = new Rate_Provider( $rate_provider, $settings[$rate_provider . '_api_token'] );
            $update_status = $provider->rates( $currency_id, $currency_code );

            if ( is_array( $update_status ) ) {
                return $update_status;
            }

            return [
                'status'  => 'success',
                'data'    => Currency::list( 'publish, draft' ),
                'message' => esc_html__( 'Currency rates updated successfully!', 'x-currency' )
            ];

        } else {
            return [
                'status'  => 'failed',
                'message' => esc_html__( 'Please go to the global settings page and place your Currency Rate Aggregator API token', 'x-currency' )
            ];
        }
    }
}
