<?php

namespace X_Currency\App\Supports\Rate_Provider;

use X_Currency;
use X_Currency\App\Utils\Currency;

class Rate_Provider
{

    /**
     * @var mixed
     */
    public $provider;
    /**
     * @var mixed
     */
    public $api_token;

    /**
     * @param $provider
     * @param $api_token
     */
    public function __construct( $provider = 'fixer', $api_token = '' )
    {
        $this->provider  = $provider;
        $this->api_token = str_replace(' ', '', $api_token);
    }

    /**
     * @param $currency_id
     * @param $currency_code
     * @return mixed
     */
    public function rates( $currency_id = 'all', $currency_code = null )
    {

        $provider_list    = $this->provider_list();
        $class            = $provider_list[$this->provider]['class'];
        $base_currency_id = get_option( Currency::XBASECURRENCY );

        if ( !empty( $base_currency_id ) ) {

            $base_currency_code = get_post_meta( $base_currency_id, 'code', true );

            if ( !empty( $base_currency_code ) ) {

                $provider     = new $class;
                $api_response = $provider->get_rates( $this->api_token );

                if ( empty( $api_response['status'] ) ) {

                    $api_response = self::exchange_base_currency( $base_currency_code, $api_response );

                    if ( $currency_id === 'all' ) { // update all currency
                        self::exchange_all_currency( $api_response['rates'] );
                    } else { // update single currency
                        if ( isset( $api_response['rates'][$currency_code] ) ) {
                            update_post_meta( $currency_id, 'rate', $api_response['rates'][$currency_code] );
                        }
                    }
                } else {
                    return $api_response;
                }
            }
        }

        return true;
    }

    /**
     * @param $rates
     */
    public static function exchange_all_currency( $rates )
    {

        $currencies = get_posts( ['post_type' => X_Currency::POST_TYPE, 'post_status' => 'publish, draft', 'numberposts' => -1] );

        foreach ( $currencies as $currency ) {

            $code = get_post_meta( $currency->ID, 'code', true );
            if ( isset( $rates[$code] ) ) {
                update_post_meta( $currency->ID, 'rate', $rates[$code] );
            }
        }
    }

    public function provider_list()
    {
        return [
            'fixer'             => [
                'label' => 'Fixer',
                'class' => Fixer::class
            ],
            'fixer-api-layer'   => [
                'label' => 'Fixer Api Layer',
                'class' => Fixer_Api_Layer::class
            ],
            'currencyfreaks'    => [
                'label' => 'CurrencyFreaks',
                'class' => Currency_Freaks::class
            ],
            'openexchangerates' => [
                'label' => 'OpenExchangeRates',
                'class' => Open_Exchange_Rates::class
            ]
        ];
    }

    /**
     * @param $base_currency
     * @param $api_response
     * @return mixed
     */
    public static function exchange_base_currency( $base_currency, $api_response, $update = true )
    {
        $api_response = (array) $api_response;
        if ( empty( $api_response['base'] ) ) {
            return;
        }

        $base = $api_response['base'];
        if ( $base === $base_currency ) {
            update_option( X_Currency::X_CURRENCY_RATES, serialize( $api_response ) );
            return $api_response;
        }

        $rates = (array) $api_response['rates'];
        if ( empty( $rates[$base_currency] ) ) {
            return [];
        }

        $new_base_currency_old_rate = $rates[$base_currency];
        unset( $rates[$base_currency] );
        $old_base_currency_rate = 1 / $new_base_currency_old_rate;

        $final_rates = [$base_currency => 1, $base => $old_base_currency_rate];

        foreach ( $rates as $key => $value ) {
            $final_rates[$key] = $value / $new_base_currency_old_rate;
        }
        $api_response['base']  = $base_currency;
        $api_response['rates'] = $final_rates;

        if($update) {
            update_option( X_Currency::X_CURRENCY_RATES, serialize( $api_response ) );
        }

        return $api_response;
    }
}
