<?php

namespace X_Currency\App\Supports;

use X_Currency\App\Core\Singleton;
use X_Currency\App\Supports\Rate_Provider\Rate_Provider;

class Schedule
{
    /**
     * @var mixed
     */
    public $global_settings;

    use Singleton;

    public function run()
    {
        global $x_currency;

        $this->global_settings = $x_currency['global_settings'];

        if ( isset( $this->global_settings['currency_rate_auto_update'] ) && $this->global_settings['currency_rate_auto_update'] == true && !get_transient( 'x-currency-currency-update' ) ) {
            $this->refresh_currency_rates();
            $transient_time = $this->calculate_transient_time( $this->global_settings['rate_auto_update_time_type'], $this->global_settings['rate_auto_update_time'] );
            set_transient( 'x-currency-currency-update', $transient_time, $transient_time );
        }
    }

    public function calculate_transient_time( $type, $time )
    {
        switch ( $type ) {
            case 'hour':
                $formula = 3600;
                break;
            case 'minute':
                $formula = 60;
                break;
            default:
                $formula = 1;
        }

        return $time * $formula;
    }

    public function refresh_currency_rates()
    {
        if ( !empty( $this->global_settings['default_rate_provider'] ) ) {
            $rate_provider = $this->global_settings['default_rate_provider'];
            $provider      = new Rate_Provider( $rate_provider, $this->global_settings[$rate_provider . '_api_token'] );
            $provider->rates();
        }
    }
}
