<?php

namespace X_Currency\App\Supports\Woocommerce;

use X_Currency;
use X_Currency\App\Supports\Rate_Provider\Rate_Provider;
use X_Currency\App\Utils\Currency;

defined( 'ABSPATH' ) || exit;

class Approximate_Product_Price
{
    /**
     * @var mixed
     */
    public $x_currency;
    /**
     * @var mixed
     */
    public $country_and_currency_code;

    /**
     * @var array
     */
    public $currency_meta = [];

    /**
     * @return mixed
     */
    public function __construct()
    {
        global $x_currency;
        $this->x_currency = $x_currency;

        $this->country_and_currency_code = x_currency_get_json_file_content( X_CURRENCY_DIR . '/sample-data/country-and-currency-code.json' );

        if ( $x_currency['user_geo_location']['countryCode'] ) {

            $currency_code = $this->country_and_currency_code[$x_currency['user_geo_location']['countryCode']];

            if ( $currency_code != $x_currency['selected_currency']['code'] ) {
                $currency = x_currency_get_currencies_by_meta( 'code', $currency_code );

                if ( isset( $currency[0] ) ) {
                    $this->currency_meta           = x_currency_get_currency_by_id( $currency[0]->ID );
                    $this->currency_meta['format'] = x_currency_price_format( ['position' => $this->currency_meta['position']] );
                } else {
                    $rates = get_option( X_Currency::X_CURRENCY_RATES );
                    if ( false === $rates ) {
                        $base_currency_id   = get_option( Currency::XBASECURRENCY );
                        $base_currency_code = get_post_meta( $base_currency_id, 'code', true );
                        $rates              = serialize( Rate_Provider::exchange_base_currency( $base_currency_code, x_currency_get_json_file_content( X_CURRENCY_DIR . '/sample-data/rates.json' ) ) );
                    }
                    $rates = unserialize( $rates );

                    if ( empty( $rates['rates'][$currency_code] ) ) {
                        return;
                    }

                    $symbols             = x_currency_symbols();
                    $this->currency_meta = [
                        'rate'               => $rates['rates'][$currency_code],
                        'code'               => $currency_code,
                        'symbol'             => isset( $symbols[$currency_code] ) ? $symbols[$currency_code] : $currency_code,
                        'format'             => get_woocommerce_price_format(),
                        'decimal_separator'  => wc_get_price_decimal_separator(),
                        'thousand_separator' => wc_get_price_thousand_separator(),
                        'decimal'            => wc_get_price_decimals()
                    ];
                }

                if ( !empty( $x_currency['global_settings']['approximate_product_price'] ) ) {
                    add_filter( 'woocommerce_get_price_html', function ( $price, $product ) {
                        if ( $product->is_type( 'simple' ) && is_numeric( $product->get_price() ) ) {
                            return $price . '<span class="x-currency-product-approximate-price">' . esc_html__( 'Approximately', 'x-currency' ) . ' ' . $this->approximate_price_html( $product->get_price() ) . '</span>';
                        }
                        return $price;
                    }, 10, 2 );
                }

                if ( !empty( $x_currency['global_settings']['approximate_cart_price'] ) ) {
                    add_action( 'woocommerce_cart_totals_after_order_total', [$this, 'approximate_cart_total_amount'] );
                    add_action( 'woocommerce_review_order_after_order_total', [$this, 'approximate_cart_total_amount'] );
                }
            }
        }
    }

    public function approximate_cart_total_amount()
    {
        $wc_cart = WC()->cart;
        //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo '<tr class="x-currency-cart-approximate-total"><th>' . esc_html__( 'Approximately Total', 'woocommerce' ) . '</th><td>' . $this->approximate_price_html( $wc_cart->get_cart_contents_total() + $wc_cart->get_shipping_total() ) . '</td></tr>';
    }

    /**
     * @param $price
     */
    public function approximate_price_html( $price )
    {
        $exchanged_currency = $this->x_currency['selected_currency']['rate'];
        $product_base_price = ( 1 / $exchanged_currency ) * $price;
        $price              = $product_base_price * $this->currency_meta['rate'];
        $price              = number_format( (float) $price, intval( $this->currency_meta['decimal'] ), $this->currency_meta['decimal_separator'], $this->currency_meta['thousand_separator'] );
        $formatted_price    = sprintf( $this->currency_meta['format'], '<span class="woocommerce-Price-currencySymbol">' . $this->currency_meta['symbol'] . '</span>', $price );
        return '<span class="woocommerce-Price-amount amount"><bdi>' . $formatted_price . '</bdi></span>';
    }
}
