<?php

namespace X_Currency\App\Supports\Woocommerce;

defined( 'ABSPATH' ) || exit;

class Report
{
    /**
     * @var mixed
     */
    public $base_currency_code;

    /**
     * @return mixed
     */
    public function __construct()
    {
        add_filter( 'woocommerce_reports_get_order_report_data_args', [$this, 'report_data_args'] );
        add_filter( 'woocommerce_currency_symbol', [$this, 'currency_symbol'] );
        add_filter( 'wc_reports_tabs', [$this, 'report_switcher'] );
    }

    /**
     * @param $symbol
     * @return mixed
     */
    public function currency_symbol( $symbol )
    {
        if(empty($_REQUEST['x_currency_nonce']) || !wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['x_currency_nonce'] ) ), 'x_currency_nonce')) {
			$symbol;
		}

        $symbols = get_woocommerce_currency_symbols();
        if ( !empty( $_GET['x-currency'] ) ) {
            $symbol = isset( $symbols[$_GET['x-currency']] ) ? $symbols[sanitize_text_field(wp_unslash($_GET['x-currency']))] : $symbol;
        } else {
            $symbol = isset( $symbols[$this->base_currency_code] ) ? $symbols[$this->base_currency_code] : $symbol;
        }
        return $symbol;
    }

    public function report_switcher()
    {
        global $x_currency;
        $selected_currency_code = x_currency_get_base_currency_code();
        if(!empty($_REQUEST['x_currency_nonce']) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['x_currency_nonce'] ) ), 'x_currency_nonce') && !empty($_GET['x-currency'])) {
            $selected_currency_code = sanitize_text_field( wp_unslash( $_GET['x-currency'] ) );
        }
    ?>
    <div style="float: right;">
        <select name="x-currency" id="x-currency">
            <?php foreach($x_currency['currencies'] as $currency):?>
            <option value="<?php echo esc_attr($currency['code']) ?>" <?php selected($currency['code'], $selected_currency_code, true) ?>><?php echo esc_html($currency['name'])?></option>
            <?php endforeach;?>
        </select>
    </div>
    <script>
        jQuery(document).ready(function($) {
            $('#x-currency').change(function() {
                let currency = $(this).val();
                let url = new URL(window.location.href);
                url.searchParams.set('x-currency', currency);
                url.searchParams.set('x_currency_nonce', "<?php echo esc_html(wp_create_nonce('x_currency_nonce'))?>");
                window.location = url.href;
            });
        });
    </script>
    <?php
    }

    /**
     * @param $args
     * @return mixed
     */
    public function report_data_args( $args )
    {
        $selected_currency_code = x_currency_get_base_currency_code();
        if(!empty($_GET['x_currency_nonce']) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['x_currency_nonce'] ) ), 'x_currency_nonce') && !empty($_GET['x-currency'])) {
            $selected_currency_code = sanitize_text_field( wp_unslash( $_GET['x-currency'] ) );
        }

        $args['where_meta'] = [
            [
                'meta_key'   => '_order_currency',
                'operator'   => 'in',
                'meta_value' => [$selected_currency_code]
            ]
        ];
        return $args;
    }
}
