<?php

namespace X_Currency\App\Supports\Woocommerce;

use X_Currency\App\Utils\Currency;

defined( 'ABSPATH' ) || exit;

class Settings
{
    /**
     * @var array
     */
    public $global_settings = [];
    /**
     * @var mixed
     */
    public $selected_currency_settings;

    public function __construct()
    {
        if ( !session_id() ) {
            @session_start();
        }

        global $x_currency;

        $x_currency['global_settings']   = x_currency_global_settings();
        $ip                              = $this->get_user_ip_address();
        $user_geo_location               = wp_remote_get( 'http://ip-api.com/json/' . $ip );
        $user_geo_location               = wp_remote_retrieve_body( $user_geo_location );
        $x_currency['user_geo_location'] = json_decode( $user_geo_location, true );

        if ( empty( $x_currency['user_geo_location']['countryCode'] ) ) {
            $x_currency['user_geo_location']['countryCode'] = false;
        }

        $this->select_currency( $x_currency['user_geo_location']['countryCode'] );

        $x_currency['selected_currency'] = $this->selected_currency_settings;
        $x_currency['currencies']        = Currency::list('publish, draft');
    }

    /**
     * @return null
     */
    public function select_currency( $user_country )
    {
        global $x_currency;

        if ( !empty( $_GET['currency'] ) ) { //phpcs:ignore WordPress.Security.NonceVerification.Recommended --Here we are getting data from public URL, there is no possible way to verify nonce.

            //phpcs:ignore WordPress.Security.NonceVerification.Recommended --Here we are getting data from public URL, there is no possible way to verify nonce.
            $currency_id = x_currency_get_currency_id_by_meta( 'code', sanitize_text_field( wp_unslash( $_GET['currency'] ) ) );

            if ( $currency_id !== false && $this->set_currency( $currency_id, $user_country ) ) {
                return;
            }
        }

        if ( !empty( $_SESSION['x_currency_code'] ) ) {

            $currency_id = x_currency_get_currency_id_by_meta( 'code', sanitize_text_field( $_SESSION['x_currency_code'] ) );

            if ( $currency_id !== false && $this->set_currency( $currency_id, $user_country ) ) {
                return;
            }
        }

        if ( isset( $x_currency['global_settings']['user_welcome_currency_type'] ) ) {

            if ( $x_currency['global_settings']['user_welcome_currency_type'] === 'auto' ) {

                if ( !empty( $user_country ) ) {

                    $currency_id = x_currency_get_currency_id_by_meta( 'welcome_country', $user_country );

                    if ( $currency_id !== false && $this->set_currency( $currency_id, $user_country ) ) {
                        return;
                    }
                }

            } else {
                if ( !empty( $x_currency['global_settings']['user_welcome_currency'] ) ) {
                    if ( !empty( get_post( $x_currency['global_settings']['user_welcome_currency'] ) ) ) {
                        $this->set_currency( $x_currency['global_settings']['user_welcome_currency'], $user_country );
                        return;
                    }
                }
            }
        }

        $base_currency_id = get_option( Currency::XBASECURRENCY );

        if ( !empty( $base_currency_id ) ) {
            $this->set_currency( intval( $base_currency_id ), $user_country );
        }
        return;
    }

    /**
     * @param $currency_id
     */
    public function set_currency( $currency_id, $user_country )
    {
        $this->selected_currency_settings = x_currency_get_currency_by_id( $currency_id );

        if ( $user_country ) {
            if ( isset( $this->selected_currency_settings['disable_countries'] ) && is_array( $this->selected_currency_settings['disable_countries'] ) ) {

                if ( $this->selected_currency_settings['geo_countries_type'] == 'disable' ) {
                    if ( in_array( $user_country, $this->selected_currency_settings['disable_countries'] ) ) {
                        return false;
                    }
                } else {
                    if ( !in_array( $user_country, $this->selected_currency_settings['disable_countries'] ) ) {
                        return false;
                    }
                }
            }
        }

        $_SESSION['x_currency_code'] = sanitize_text_field( $this->selected_currency_settings['code'] );
        return true;
    }

    /**
     * @return mixed
     */
    public function get_user_ip_address()
    {
        foreach ( ['HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR'] as $key ) {
            if ( array_key_exists( $key, $_SERVER ) === true ) {
                foreach ( explode( ',', sanitize_text_field( wp_unslash( $_SERVER[$key] ) ) ) as $ip ) {
                    $ip = trim( $ip );
                    if ( filter_var( $ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE ) !== false ) {
                        return $ip;
                    }
                }
            }
        }
        return '';
    }
}
