<?php

namespace X_Currency\Bootstrap;

class Application
{
    private static $instance;

    /**
     * @return static
     */
    public static function getInstance()
    {
        if ( !self::$instance ) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function boot()
    {
        if ( !$this->is_compatible() ) {
            add_action( 'admin_notices', [$this, 'admin_notice_missing_main_plugin'] );
            return;
        }

        add_action('init', function() : void {
            load_plugin_textdomain( "x-currency", false, X_CURRENCY_DIR. "languages" );
        } );

        $config = include_once X_CURRENCY_DIR . 'config/app.php';
        foreach ( $config['providers'] as $provider_class ) {
            $provider = new $provider_class();
            if ( $provider instanceof Service_Provider ) {
                $provider->boot();
            }
        }
    }

    public function is_compatible()
    {
        $plugin_path = trailingslashit( WP_PLUGIN_DIR ) . 'woocommerce/woocommerce.php';
        if ( in_array( $plugin_path, wp_get_active_and_valid_plugins() ) ) {
            return true;
        }
        return false;
    }

    public function admin_notice_missing_main_plugin()
    {
        $btn = [];
        if(file_exists(WP_PLUGIN_DIR . '/woocommerce/woocommerce.php')) {
			$btn['label'] = esc_html__('Activate WooCommerce', 'x-currency');
			$btn['url']   = wp_nonce_url('plugins.php?action=activate&plugin=woocommerce/woocommerce.php&plugin_status=all&paged=1', 'activate-plugin_woocommerce/woocommerce.php');
		} else {
			$btn['label'] = esc_html__('Install WooCommerce', 'x-currency');
			$btn['url']   = wp_nonce_url(self_admin_url('update.php?action=install-plugin&plugin=woocommerce'), 'install-plugin_woocommerce');
		}
        ?>
        <div class="notice notice-error is-dismissible" style="padding-bottom: 10px;">
            <p><?php esc_html_e('X-Currency requires the WooCommerce plugin, which is not currently running.', 'x-currency')?></p>
            <a href="<?php echo esc_url($btn['url']) ?>" class="button-primary"><?php echo esc_html($btn['label'])?></a>
        </div>
        <?php
    }
}
