<?php

namespace X_Currency\Routes;

use X_Currency\App\Controllers\Currency_Controller;
use X_Currency\App\Controllers\Geoip_Controller;
use X_Currency\App\Controllers\Settings_Controller;
use X_Currency\App\Controllers\Switcher_Controller;
use X_Currency\App\Utils\Currency;
use X_Currency\App\Utils\Settings;
use X_Currency\App\Core\Api as ApiBase;
use X_Currency\App\Supports\Rate_Provider\Rate_Exchange;

class Api extends ApiBase
{
    public function config()
    {
        $this->prefix = 'admin-settings';
    }

    /**
     * @return mixed
     */
    public function get_currencies()
    {
        return [
            'message' => esc_html__( 'Currencies retrieved successfully!', 'x-currency' ),
            'data'    => Currency::list( 'publish, draft' ),
            'status'  => 'success'
        ];
    }

    /**
     * @return mixed
     */
    public function get_currency_input_fields()
    {
        return ( new Currency_Controller )->currency_input_fields();
    }

    /**
     * @return mixed
     */
    public function post_create_currency()
    {
        return ( new Currency_Controller )->create_currency( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_update_currency()
    {
        return ( new Currency_Controller )->update_currency( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_currency_organizer()
    {
        return ( new Currency_Controller )->currency_organizer( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function get_setting_inputs()
    {
        return ( new Settings_Controller )->setting_inputs();
    }

    public function get_global_settings()
    {
        return ['data' => Settings::settings_data(), 'message' => ''];
    }

    /**
     * @return mixed
     */
    public function post_save_settings()
    {
        return ( new Settings_Controller )->save_settings( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_attachment()
    {
        return ( new Currency_Controller )->get_attachment( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_update_base_currency()
    {
        return ( new Currency_Controller )->update_base_currency( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function get_count_product()
    {
        return ( new Currency_Controller )->count_product();
    }

    /**
     * @return mixed
     */
    public function get_switcher_list()
    {
        return ( new Switcher_Controller )->switcher_list();
    }

    /**
     * @return mixed
     */
    public function post_create_switcher()
    {
        return ( new Switcher_Controller )->create_switcher( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_update_switcher()
    {
        return ( new Switcher_Controller )->update_switcher( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_switcher_organizer()
    {
        return ( new Switcher_Controller )->switcher_organizer( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function post_exchange_single()
    {
        return ( new Rate_Exchange )->exchange_single( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function get_exchange_all()
    {
        return ( new Rate_Exchange )->exchange_all();
    }

    /**
     * @return mixed
     */
    public function post_save_currency_geo_locations()
    {
        return ( new Geoip_Controller )->save_geo_locations( $this->request->get_params() );
    }

    /**
     * @return mixed
     */
    public function get_geo_input_fields()
    {
        return ( new Geoip_Controller )->geo_input_fields( $this->request->get_params() );
    }

    public function get_demo_currencies()
    {
        return ( new Currency_Controller )->demo_currency_data( $this->request->get_params() );
    }

    public function get_pages()
    {
        return ( new Switcher_Controller )->get_pages();
    }
}
